@@ -37,6 +37,7 @@
 #include <QGuiApplication>
 #include <QSortFilterProxyModel>
 #include <QTimer>
+#include <QSessionManager>
 
 // KDE includes
 #include <KAboutApplicationDialog>
@@ -68,6 +69,7 @@
 class OurSortFilterProxyModel : public QSortFilterProxyModel, public QQmlParserStatus
 {
     Q_OBJECT
+    Q_INTERFACES(QQmlParserStatus)
 public:
     void classBegin() override {}
     void componentComplete() override {
@@ -154,13 +156,18 @@ QStringList DiscoverObject::modes() const
 
 void DiscoverObject::openMode(const QString& _mode)
 {
+    QObject* obj = rootObject();
+    if (!obj) {
+        qWarning() << "could not get the main object";
+        return;
+    }
+
     if(!modes().contains(_mode))
         qWarning() << "unknown mode" << _mode;
 
     QString mode = _mode;
     mode[0] = mode[0].toUpper();
 
-    QObject* obj = rootObject();
     const QByteArray propertyName = "top"+mode.toLatin1()+"Comp";
     const QVariant modeComp = obj->property(propertyName.constData());
     if (!modeComp.isValid())
@@ -235,7 +242,7 @@ void DiscoverObject::openApplication(const QUrl& url)
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
             connect(stream, &StoredResultsStream::finished, this, [this, url, stream]() {
                 const auto res = stream->resources();
-                if (res.count() == 1) {
+                if (res.count() >= 1) {
                     emit openApplicationInternal(res.first());
                 } else {
                     setRootObjectProperty("defaultStartup", true);
@@ -274,6 +281,13 @@ void DiscoverObject::integrateObject(QObject* object)
 
     object->installEventFilter(this);
     connect(object, &QObject::destroyed, qGuiApp, &QCoreApplication::quit);
+
+    connect(qGuiApp, &QGuiApplication::commitDataRequest, this, [this](QSessionManager &sessionManager) {
+        if (ResourcesModel::global()->isBusy()) {
+            Q_EMIT preventedClose();
+            sessionManager.cancel();
+        }
+    });
 }
 
 bool DiscoverObject::eventFilter(QObject * object, QEvent * event)
