@@ -58,7 +58,6 @@ void CategoryModel::populateCategories()
                 Category::addSubcategory(ret, c);
         }
     }
-    qSort(ret.begin(), ret.end(), Category::categoryLessThan);
     if (m_rootCategories != ret) {
         m_rootCategories = ret;
         Q_EMIT rootCategoriesChanged();
@@ -72,13 +71,9 @@ QVariantList CategoryModel::rootCategoriesVL() const
 
 void CategoryModel::blacklistPlugin(const QString &name)
 {
-    const QSet<QString> plugins = {name};
-    for(auto it = m_rootCategories.begin(), itEnd = m_rootCategories.end(); it!=itEnd; ) {
-        if ((*it)->blacklistPlugins(plugins)) {
-            delete *it;
-            it = m_rootCategories.erase(it);
-        } else
-            ++it;
+    const bool ret = Category::blacklistPluginsInVector({name}, m_rootCategories);
+    if (ret) {
+        Q_EMIT rootCategoriesChanged();
     }
 }
 
