@@ -24,6 +24,7 @@
 #include <QAbstractListModel>
 #include "discovercommon_export.h"
 
+class QTimer;
 class ResourcesUpdatesModel;
 class AbstractResource;
 class UpdateItem;
@@ -64,7 +65,7 @@ public:
     bool hasUpdates() const;
 
     ///all upgradeable packages
-    int totalUpdatesCount() const { return m_updateItems.count(); }
+    int totalUpdatesCount() const;
 
     ///packages marked to upgrade
     int toUpdateCount() const;
@@ -91,6 +92,7 @@ private:
     void resourceHasProgressed(AbstractResource* res, qreal progress);
     void activityChanged();
 
+    QTimer* const m_updateSizeTimer;
     QVector<UpdateItem*> m_updateItems;
     ResourcesUpdatesModel* m_updates;
     QList<AbstractResource*> m_resources;
