@@ -92,25 +92,34 @@ void PKTransaction::trigger(PackageKit::Transaction::TransactionFlags flags)
     connect(m_trans.data(), &PackageKit::Transaction::errorCode, this, &PKTransaction::errorFound);
     connect(m_trans.data(), &PackageKit::Transaction::mediaChangeRequired, this, &PKTransaction::mediaChange);
     connect(m_trans.data(), &PackageKit::Transaction::requireRestart, this, &PKTransaction::requireRestart);
-    connect(m_trans.data(), &PackageKit::Transaction::itemProgress, this, &PKTransaction::progressChanged);
+    connect(m_trans.data(), &PackageKit::Transaction::repoSignatureRequired, this, &PKTransaction::repoSignatureRequired);
+    connect(m_trans.data(), &PackageKit::Transaction::percentageChanged, this, &PKTransaction::progressChanged);
+    connect(m_trans.data(), &PackageKit::Transaction::statusChanged, this, &PKTransaction::statusChanged);
     connect(m_trans.data(), &PackageKit::Transaction::eulaRequired, this, &PKTransaction::eulaRequired);
     connect(m_trans.data(), &PackageKit::Transaction::allowCancelChanged, this, &PKTransaction::cancellableChanged);
     
     setCancellable(m_trans->allowCancel());
 }
 
-void PKTransaction::progressChanged(const QString &id, PackageKit::Transaction::Status status, uint percentage)
+void PKTransaction::statusChanged()
 {
-    PackageKitResource * res = qobject_cast<PackageKitResource*>(resource());
-    if (!res->allPackageNames().contains(PackageKit::Daemon::packageName(id)))
-        return;
+    setStatus(m_trans->status() == PackageKit::Transaction::StatusDownload ? Transaction::DownloadingStatus : Transaction::CommittingStatus);
+    progressChanged();
+}
 
-    setProgress(percentage);
+int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage);
 
-    if (status == PackageKit::Transaction::StatusDownload)
-        setStatus(Transaction::DownloadingStatus);
-    else
-        setStatus(Transaction::CommittingStatus);
+void PKTransaction::progressChanged()
+{
+    auto percent = m_trans->percentage();
+    if (percent == 101) {
+        qWarning() << "percentage cannot be calculated";
+        percent = 50;
+    }
+
+    const auto processedPercentage = percentageWithStatus(m_trans->status(), qBound<int>(0, percent, 100));
+    if (processedPercentage >= 0)
+        setProgress(processedPercentage);
 }
 
 void PKTransaction::cancellableChanged()
@@ -132,7 +141,8 @@ void PKTransaction::cancel()
 void PKTransaction::cleanup(PackageKit::Transaction::Exit exit, uint runtime)
 {
     Q_UNUSED(runtime)
-    const bool cancel = exit == PackageKit::Transaction::ExitEulaRequired || exit == PackageKit::Transaction::ExitCancelled;
+    const bool cancel = !m_proceedFunctions.isEmpty() || exit == PackageKit::Transaction::ExitCancelled;
+    const bool failed = exit == PackageKit::Transaction::ExitFailed;
     const bool simulate = m_trans->transactionFlags() & PackageKit::Transaction::TransactionFlagSimulate;
 
     disconnect(m_trans, nullptr, this, nullptr);
@@ -140,7 +150,7 @@ void PKTransaction::cleanup(PackageKit::Transaction::Exit exit, uint runtime)
 
     const auto backend = qobject_cast<PackageKitBackend*>(resource()->backend());
 
-    if (!cancel && simulate) {
+    if (!cancel && !failed && simulate) {
         auto packagesToRemove = m_newPackageStates.value(PackageKit::Transaction::InfoRemoving);
         QMutableListIterator<QString> i(packagesToRemove);
         QSet<AbstractResource*> removedResources;
@@ -154,14 +164,15 @@ void PKTransaction::cleanup(PackageKit::Transaction::Exit exit, uint runtime)
         }
         removedResources.subtract(m_apps.toList().toSet());
 
-        QString msg = PackageKitResource::joinPackages(packagesToRemove);
-        if (!removedResources.isEmpty()) {
-            const QStringList removedResourcesStr = kTransform<QStringList>(removedResources, [](AbstractResource* a) { return a->name(); });
-            msg += QLatin1Char('\n');
-            msg += removedResourcesStr.join(QStringLiteral(", "));
-        }
+        if (!packagesToRemove.isEmpty() || !removedResources.isEmpty()) {
+            QString msg = QStringLiteral("<ul><li>") + PackageKitResource::joinPackages(packagesToRemove, QStringLiteral("</li><li>"));
+            if (!removedResources.isEmpty()) {
+                const QStringList removedResourcesStr = kTransform<QStringList>(removedResources, [](AbstractResource* a) { return a->name(); });
+                msg += QLatin1Char('\n');
+                msg += removedResourcesStr.join(QStringLiteral("</li><li>"));
+            }
+            msg += QStringLiteral("</li></ul>");
 
-        if (!msg.isEmpty()) {
             Q_EMIT proceedRequest(i18n("Confirm..."), i18np("To proceed with this action, the following package needs removal:\n%2", "To proceed with this action, the following packages need removal:\n%2", packagesToRemove.count(), msg));
         } else {
             proceed();
@@ -170,17 +181,37 @@ void PKTransaction::cleanup(PackageKit::Transaction::Exit exit, uint runtime)
     }
 
     this->submitResolve();
-    setStatus(Transaction::CancelledStatus);
+    if (failed)
+        setStatus(Transaction::DoneWithErrorStatus);
+    else
+        setStatus(Transaction::CancelledStatus);
+}
+
+void PKTransaction::processProceedFunction()
+{
+    auto t = m_proceedFunctions.takeFirst()();
+    connect(t, &PackageKit::Transaction::finished, this, [this](PackageKit::Transaction::Exit status) {
+        if (status != PackageKit::Transaction::Exit::ExitSuccess) {
+            qWarning() << "transaction failed" << sender() << status;
+            cancel();
+            return;
+        }
+
+        if (!m_proceedFunctions.isEmpty()) {
+            processProceedFunction();
+        } else {
+            start();
+        }
+    });
 }
 
 void PKTransaction::proceed()
 {
-    if (!m_requiredEula.isEmpty()) {
-        PackageKit::Transaction* t = PackageKit::Daemon::acceptEula(m_requiredEula.takeFirst());
-        connect(t, &PackageKit::Transaction::finished, this, &PKTransaction::start);
-        return;
+    if (!m_proceedFunctions.isEmpty()) {
+        processProceedFunction();
+    } else {
+        trigger(PackageKit::Transaction::TransactionFlagOnlyTrusted);
     }
-    trigger(PackageKit::Transaction::TransactionFlagOnlyTrusted);
 }
 
 void PKTransaction::packageResolved(PackageKit::Transaction::Info info, const QString& packageId)
@@ -191,13 +222,15 @@ void PKTransaction::packageResolved(PackageKit::Transaction::Info info, const QS
 void PKTransaction::submitResolve()
 {
     QStringList needResolving;
-    const auto pkgids = m_newPackageStates.value(PackageKit::Transaction::InfoFinished);
-    foreach(const auto pkgid, pkgids) {
-        needResolving += PackageKit::Daemon::packageName(pkgid);
+    foreach(const auto &pkgids, m_newPackageStates) {
+        foreach(const auto &pkgid, pkgids) {
+            needResolving += PackageKit::Daemon::packageName(pkgid);
+        }
     }
-    const auto backend = qobject_cast<PackageKitBackend*>(resource()->backend());
 
     if (!needResolving.isEmpty()) {
+        needResolving.removeDuplicates();
+        const auto backend = qobject_cast<PackageKitBackend*>(resource()->backend());
         backend->clearPackages(needResolving);
         backend->resolvePackages(needResolving);
     }
@@ -210,7 +243,10 @@ PackageKit::Transaction* PKTransaction::transaction()
 
 void PKTransaction::eulaRequired(const QString& eulaID, const QString& packageID, const QString& vendor, const QString& licenseAgreement)
 {
-    m_requiredEula += eulaID;
+    m_proceedFunctions << [eulaID](){
+        return PackageKit::Daemon::acceptEula(eulaID);
+    };
+
     Q_EMIT proceedRequest(i18n("Accept EULA"), i18n("The package %1 and its vendor %2 require that you accept their license:\n %3",
                                                  PackageKit::Daemon::packageName(packageID), vendor, licenseAgreement));
 }
@@ -233,3 +269,16 @@ void PKTransaction::requireRestart(PackageKit::Transaction::Restart restart, con
 {
     Q_EMIT passiveMessage(PackageKitMessages::restartMessage(restart, pkgid));
 }
+
+void PKTransaction::repoSignatureRequired(const QString& packageID, const QString& repoName, const QString& keyUrl,
+                                          const QString& keyUserid, const QString& keyId, const QString& keyFingerprint,
+                                          const QString& keyTimestamp, PackageKit::Transaction::SigType type)
+{
+    Q_EMIT proceedRequest(i18n("Missing signature for %1 in %2", packageID, repoName),
+                          i18n("Do you trust the following key?\n\nUrl: %1\nUser: %2\nKey: %3\nFingerprint: %4\nTimestamp: %4\n",
+                               keyUrl, keyUserid, keyFingerprint, keyTimestamp));
+
+    m_proceedFunctions << [type, keyId, packageID](){
+        return PackageKit::Daemon::installSignature(type, keyId, packageID);
+    };
+}
