@@ -53,6 +53,18 @@
 
 DISCOVER_BACKEND_PLUGIN(PackageKitBackend)
 
+template <typename T, typename W>
+static void setWhenAvailable(const QDBusPendingReply<T>& pending, W func, QObject* parent)
+{
+    QDBusPendingCallWatcher* watcher = new QDBusPendingCallWatcher(pending, parent);
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished,
+                    parent, [func](QDBusPendingCallWatcher* watcher) {
+                        watcher->deleteLater();
+                        QDBusPendingReply<T> reply = *watcher;
+                        func(reply.value());
+                    });
+}
+
 QString PackageKitBackend::locateService(const QString &filename)
 {
     return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("applications/")+filename);
@@ -60,13 +72,14 @@ QString PackageKitBackend::locateService(const QString &filename)
 
 PackageKitBackend::PackageKitBackend(QObject* parent)
     : AbstractResourcesBackend(parent)
+    , m_appdata(new AppStream::Pool)
     , m_updater(new PackageKitUpdater(this))
     , m_refresher(nullptr)
     , m_isFetching(0)
     , m_reviews(AppStreamIntegration::global()->reviews())
 {
     QTimer* t = new QTimer(this);
-    connect(t, &QTimer::timeout, this, &PackageKitBackend::refreshDatabase);
+    connect(t, &QTimer::timeout, this, &PackageKitBackend::checkForUpdates);
     t->setInterval(60 * 60 * 1000);
     t->setSingleShot(false);
     t->start();
@@ -81,17 +94,12 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
 
     SourcesModel::global()->addSourcesBackend(new PackageKitSourcesBackend(this));
 
-    QString error;
-    const bool b = m_appdata.load(&error);
     reloadPackageList();
 
-    if (!b && m_packages.packages.isEmpty()) {
-        qWarning() << "Could not open the AppStream metadata pool" << error;
-
-        QTimer::singleShot(0, this, [this]() {
-            Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
-        });
-    }
+    setWhenAvailable(PackageKit::Daemon::getTimeSinceAction(PackageKit::Transaction::RoleRefreshCache), [this](uint timeSince) {
+        if (timeSince > 3600)
+            checkForUpdates();
+    }, this);
 }
 
 PackageKitBackend::~PackageKitBackend() = default;
@@ -121,7 +129,18 @@ void PackageKitBackend::reloadPackageList()
         disconnect(m_refresher.data(), &PackageKit::Transaction::finished, this, &PackageKitBackend::reloadPackageList);
     }
 
-    const auto components = m_appdata.components();
+    QString error;
+    m_appdata.reset(new AppStream::Pool);
+    const bool b = m_appdata->load(&error);
+    if (!b && m_packages.packages.isEmpty()) {
+        qWarning() << "Could not open the AppStream metadata pool" << error;
+
+        QTimer::singleShot(0, this, [this]() {
+            Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
+        });
+    }
+
+    const auto components = m_appdata->components();
     QStringList neededPackages;
     neededPackages.reserve(components.size());
     foreach(const AppStream::Component& component, components) {
@@ -130,8 +149,9 @@ void PackageKitBackend::reloadPackageList()
 
         const auto pkgNames = component.packageNames();
         if (pkgNames.isEmpty()) {
-            if (component.kind() == AppStream::Component::KindDesktopApp) {
-                const QString file = locateService(component.desktopId());
+            auto launchable = component.launchable(AppStream::Launchable::KindDesktopId);
+            if (component.kind() == AppStream::Component::KindDesktopApp && !launchable.entries().isEmpty()) {
+                const QString file = locateService(launchable.entries().constFirst());
                 if (!file.isEmpty()) {
                     auto trans = PackageKit::Daemon::searchFiles(file);
                     connect(trans, &PackageKit::Transaction::package, this, [trans](PackageKit::Transaction::Info info, const QString &packageID){
@@ -144,6 +164,7 @@ void PackageKitBackend::reloadPackageList()
                             const auto pkgid = pkgidVal.toString();
                             acquireFetching(true);
                             auto res = addComponent(component, {PackageKit::Daemon::packageName(pkgid)});
+                            res->clearPackageIds();
                             res->addPackageId(PackageKit::Transaction::InfoInstalled, pkgid, true);
                             acquireFetching(false);
                         }
@@ -161,8 +182,15 @@ void PackageKitBackend::reloadPackageList()
     }
 
     acquireFetching(false);
-    neededPackages.removeDuplicates();
-    resolvePackages(neededPackages);
+    if (!neededPackages.isEmpty()) {
+        neededPackages.removeDuplicates();
+        resolvePackages(neededPackages);
+    } else {
+        qDebug() << "empty appstream db";
+        if (PackageKit::Daemon::backendName() == QLatin1String("aptcc") || PackageKit::Daemon::backendName().isEmpty()) {
+            checkForUpdates();
+        }
+    }
 }
 
 AppPackageKitResource* PackageKitBackend::addComponent(const AppStream::Component& component, const QStringList& pkgNames)
@@ -170,8 +198,14 @@ AppPackageKitResource* PackageKitBackend::addComponent(const AppStream::Componen
     Q_ASSERT(isFetching());
     Q_ASSERT(!pkgNames.isEmpty());
 
-    const auto res = new AppPackageKitResource(component, pkgNames.at(0), this);
-    m_packages.packages[component.id()] = res;
+
+    AppPackageKitResource* res = qobject_cast<AppPackageKitResource*>(m_packages.packages[component.id()]);
+    if (!res) {
+        res = new AppPackageKitResource(component, pkgNames.at(0), this);
+        m_packages.packages[component.id()] = res;
+    } else {
+        res->clearPackageIds();
+    }
     foreach (const QString& pkg, pkgNames) {
         m_packages.packageToApp[pkg] += component.id();
     }
@@ -323,17 +357,13 @@ T PackageKitBackend::resourcesByPackageNames(const Q &pkgnames) const
 }
 
 void PackageKitBackend::checkForUpdates()
-{
-    refreshDatabase();
-}
-
-void PackageKitBackend::refreshDatabase()
 {
     if (!m_refresher) {
         acquireFetching(true);
         m_refresher = PackageKit::Daemon::refreshCache(false);
         connect(m_refresher.data(), &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
         connect(m_refresher.data(), &PackageKit::Transaction::finished, this, [this]() {
+            m_refresher = nullptr;
             reloadPackageList();
             acquireFetching(false);
         });
@@ -344,7 +374,7 @@ void PackageKitBackend::refreshDatabase()
 
 QList<AppStream::Component> PackageKitBackend::componentsById(const QString& id) const
 {
-    return m_appdata.componentsById(id);
+    return m_appdata->componentsById(id);
 }
 
 ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters& filter)
@@ -357,7 +387,7 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
     } else if (filter.search.isEmpty()) {
         return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return !res->isTechnical(); }));
     } else {
-        const QList<AppStream::Component> components = m_appdata.search(filter.search);
+        const QList<AppStream::Component> components = m_appdata->search(filter.search);
         const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component& comp) { return comp.id(); });
         auto stream = new ResultsStream(QStringLiteral("PackageKitStream-search"));
         if (!ids.isEmpty()) {
@@ -413,7 +443,7 @@ ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
                 }
             }
             if (!pkg)
-                qWarning() << "could not find" << host << deprecatedHost;
+                qDebug() << "could not find" << host << deprecatedHost;
         }
     }
     return new ResultsStream(QStringLiteral("PackageKitStream-url"), pkg ? QVector<AbstractResource*>{pkg} : QVector<AbstractResource*>{});
@@ -560,23 +590,6 @@ void PackageKitBackend::performDetailsFetch()
     PackageKit::Transaction* transaction = PackageKit::Daemon::getDetails(ids);
     connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
     connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-
-    QSharedPointer<QMap<QString, int>> packageDependencies(new QMap<QString, int>);
-    auto trans = PackageKit::Daemon::dependsOn(ids);
-    connect(trans, &PackageKit::Transaction::package, this, [packageDependencies](PackageKit::Transaction::Info /*info*/, const QString &packageID, const QString &/*summary*/) {
-        (*packageDependencies)[packageID] += 1;
-    });
-    connect(trans, &PackageKit::Transaction::finished, this, [this, packageDependencies](PackageKit::Transaction::Exit /*status*/) {
-        auto pkgDeps = (*packageDependencies);
-
-        for (auto it = pkgDeps.constBegin(), itEnd = pkgDeps.constEnd(); it != itEnd; ++it) {
-            const auto resources = resourcesByPackageName(PackageKit::Daemon::packageName(it.key()));
-            for(auto resource : resources) {
-                auto pkres = qobject_cast<PackageKitResource*>(resource);
-                pkres->setDependenciesCount(it.value());
-            }
-        }
-    });
 }
 
 void PackageKitBackend::checkDaemonRunning()
