@@ -66,12 +66,20 @@ void StandardBackendUpdater::start()
     m_settingUp = true;
     emit progressingChanged(true);
     setProgress(0);
+    auto upgradeList = m_toUpgrade.toList();
+    qSort(upgradeList.begin(), upgradeList.end(), [](const AbstractResource* a, const AbstractResource* b){ return a->name() < b->name(); });
 
-    foreach(AbstractResource* res, m_toUpgrade) {
+    const bool couldCancel = m_canCancel;
+    foreach(AbstractResource* res, upgradeList) {
         m_pendingResources += res;
         auto t = m_backend->installApplication(res);
         t->setVisible(false);
+        connect(this, &StandardBackendUpdater::cancelTransaction, t, &Transaction::cancel);
         TransactionModel::global()->addTransaction(t);
+        m_canCancel |= t->isCancellable();
+    }
+    if (m_canCancel != couldCancel) {
+        Q_EMIT cancelableChanged(m_canCancel);
     }
     m_settingUp = false;
 
@@ -82,6 +90,11 @@ void StandardBackendUpdater::start()
     }
 }
 
+void StandardBackendUpdater::cancel()
+{
+    Q_EMIT cancelTransaction();
+}
+
 void StandardBackendUpdater::transactionAdded(Transaction* newTransaction)
 {
     if (!m_pendingResources.contains(newTransaction->resource()))
@@ -207,8 +220,7 @@ QDateTime StandardBackendUpdater::lastUpdate() const
 
 bool StandardBackendUpdater::isCancelable() const
 {
-    //We don't really know when we can cancel, so we never let
-    return false;
+    return m_canCancel;
 }
 
 bool StandardBackendUpdater::isProgressing() const
